@@ -6,22 +6,19 @@ Represents an unbounded float using a widget.
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from .domwidget import LabeledWidget
-from .valuewidget import ValueWidget
-from .widget import register
-from .widget_core import CoreWidget
-from .trait_types import Color
 from traitlets import (
-    Unicode, CFloat, Bool, Int, CaselessStrEnum, Tuple, TraitError, validate
+    Instance, Unicode, CFloat, Bool, CaselessStrEnum, Tuple, TraitError, validate, default
 )
+from .widget_description import DescriptionWidget
+from .trait_types import InstanceDict, NumberFormat
+from .valuewidget import ValueWidget
+from .widget import register, widget_serialization
+from .widget_core import CoreWidget
+from .widget_int import ProgressStyle, SliderStyle
 
 
-class _Float(LabeledWidget, ValueWidget, CoreWidget):
+class _Float(DescriptionWidget, ValueWidget, CoreWidget):
     value = CFloat(0.0, help="Float value").tag(sync=True)
-    disabled = Bool(False, help="Enable or disable user changes").tag(sync=True)
-
-    _model_module = Unicode('jupyter-js-widgets').tag(sync=True)
-    _view_module = Unicode('jupyter-js-widgets').tag(sync=True)
 
     def __init__(self, value=None, **kwargs):
         if value is not None:
@@ -32,7 +29,6 @@ class _Float(LabeledWidget, ValueWidget, CoreWidget):
 class _BoundedFloat(_Float):
     max = CFloat(100.0, help="Max value").tag(sync=True)
     min = CFloat(0.0, help="Min value").tag(sync=True)
-    step = CFloat(0.1, help="Minimum step to increment the value (ignored by some views)").tag(sync=True)
 
     @validate('value')
     def _validate_value(self, proposal):
@@ -62,8 +58,42 @@ class _BoundedFloat(_Float):
             self.value = max
         return max
 
+class _BoundedLogFloat(_Float):
+    max = CFloat(4.0, help="Max value for the exponent").tag(sync=True)
+    min = CFloat(0.0, help="Min value for the exponent").tag(sync=True)
+    base = CFloat(10.0, help="Base of value").tag(sync=True)
+    value = CFloat(1.0, help="Float value").tag(sync=True)
 
-@register('Jupyter.FloatText')
+    @validate('value')
+    def _validate_value(self, proposal):
+        """Cap and floor value"""
+        value = proposal['value']
+        if self.base ** self.min > value or self.base ** self.max < value:
+            value = min(max(value, self.base **  self.min), self.base **  self.max)
+        return value
+
+    @validate('min')
+    def _validate_min(self, proposal):
+        """Enforce base ** min <= value <= base ** max"""
+        min = proposal['value']
+        if min > self.max:
+            raise TraitError('Setting min > max')
+        if self.base ** min > self.value:
+            self.value = self.base ** min
+        return min
+
+    @validate('max')
+    def _validate_max(self, proposal):
+        """Enforce base ** min <= value <= base ** max"""
+        max = proposal['value']
+        if max < self.min:
+            raise TraitError('setting max < min')
+        if self.base ** max < self.value:
+            self.value = self.base ** max
+        return max
+
+
+@register
 class FloatText(_Float):
     """ Displays a float value within a textbox. For a textbox in
     which the value must be within a specific range, use BoundedFloatText.
@@ -72,16 +102,19 @@ class FloatText(_Float):
     ----------
     value : float
         value displayed
+    step : float
+        step of the increment (if None, any step is allowed)
     description : str
         description displayed next to the text box
-    color : str Unicode color code (eg. '#C13535')
-        color of the value displayed
     """
     _view_name = Unicode('FloatTextView').tag(sync=True)
     _model_name = Unicode('FloatTextModel').tag(sync=True)
+    disabled = Bool(False, help="Enable or disable user changes").tag(sync=True)
+    continuous_update = Bool(False, help="Update the value as the user types. If False, update on submission, e.g., pressing Enter or navigating away.").tag(sync=True)
+    step = CFloat(None, allow_none=True, help="Minimum step to increment the value").tag(sync=True)
 
 
-@register('Jupyter.BoundedFloatText')
+@register
 class BoundedFloatText(_BoundedFloat):
     """ Displays a float value within a textbox. Value must be within the range specified.
 
@@ -95,16 +128,18 @@ class BoundedFloatText(_BoundedFloat):
         minimal value of the range of possible values displayed
     max : float
         maximal value of the range of possible values displayed
+    step : float
+        step of the increment (if None, any step is allowed)
     description : str
         description displayed next to the textbox
-    color : str Unicode color code (eg. '#C13535')
-        color of the value displayed
     """
     _view_name = Unicode('FloatTextView').tag(sync=True)
-    _model_name = Unicode('FloatTextModel').tag(sync=True)
+    _model_name = Unicode('BoundedFloatTextModel').tag(sync=True)
+    disabled = Bool(False, help="Enable or disable user changes").tag(sync=True)
+    continuous_update = Bool(False, help="Update the value as the user types. If False, update on submission, e.g., pressing Enter or navigating away.").tag(sync=True)
+    step = CFloat(None, allow_none=True, help="Minimum step to increment the value").tag(sync=True)
 
-
-@register('Jupyter.FloatSlider')
+@register
 class FloatSlider(_BoundedFloat):
     """ Slider/trackbar of floating values with the specified range.
 
@@ -128,23 +163,64 @@ class FloatSlider(_BoundedFloat):
         default is '.2f', specifier for the format function used to represent
         slider value for human consumption, modeled after Python 3's format
         specification mini-language (PEP 3101).
-    slider_color : str Unicode color code (eg. '#C13535')
-        color of the slider
-    color : str Unicode color code (eg. '#C13535')
-        color of the value displayed (if readout == True)
     """
     _view_name = Unicode('FloatSliderView').tag(sync=True)
     _model_name = Unicode('FloatSliderModel').tag(sync=True)
+    step = CFloat(0.1, help="Minimum step to increment the value").tag(sync=True)
     orientation = CaselessStrEnum(values=['horizontal', 'vertical'],
         default_value='horizontal', help="Vertical or horizontal.").tag(sync=True)
-    _range = Bool(False, help="Display a range selector").tag(sync=True)
     readout = Bool(True, help="Display the current value of the slider next to it.").tag(sync=True)
-    readout_format = Unicode('.2f', help="Format for the readout").tag(sync=True)
-    slider_color = Color(None, allow_none=True).tag(sync=True)
+    readout_format = NumberFormat(
+        '.2f', help="Format for the readout").tag(sync=True)
     continuous_update = Bool(True, help="Update the value of the widget as the user is holding the slider.").tag(sync=True)
+    disabled = Bool(False, help="Enable or disable user changes").tag(sync=True)
+
+    style = InstanceDict(SliderStyle).tag(sync=True, **widget_serialization)
 
 
-@register('Jupyter.FloatProgress')
+@register
+class FloatLogSlider(_BoundedLogFloat):
+    """ Slider/trackbar of logarithmic floating values with the specified range.
+
+    Parameters
+    ----------
+    value : float
+        position of the slider
+    base : float
+        base of the logarithmic scale. Default is 10
+    min : float
+        minimal position of the slider in log scale, i.e., actual minimum is base ** min
+    max : float
+        maximal position of the slider in log scale, i.e., actual maximum is base ** max
+    step : float
+        step of the trackbar, denotes steps for the exponent, not the actual value
+    description : str
+        name of the slider
+    orientation : {'horizontal', 'vertical'}
+        default is 'horizontal', orientation of the slider
+    readout : {True, False}
+        default is True, display the current value of the slider next to it
+    readout_format : str
+        default is '.3g', specifier for the format function used to represent
+        slider value for human consumption, modeled after Python 3's format
+        specification mini-language (PEP 3101).
+    """
+    _view_name = Unicode('FloatLogSliderView').tag(sync=True)
+    _model_name = Unicode('FloatLogSliderModel').tag(sync=True)
+    step = CFloat(0.1, help="Minimum step in the exponent to increment the value").tag(sync=True)
+    orientation = CaselessStrEnum(values=['horizontal', 'vertical'],
+        default_value='horizontal', help="Vertical or horizontal.").tag(sync=True)
+    readout = Bool(True, help="Display the current value of the slider next to it.").tag(sync=True)
+    readout_format = NumberFormat(
+        '.3g', help="Format for the readout").tag(sync=True)
+    continuous_update = Bool(True, help="Update the value of the widget as the user is holding the slider.").tag(sync=True)
+    disabled = Bool(False, help="Enable or disable user changes").tag(sync=True)
+    base = CFloat(10., help="Base for the logarithm").tag(sync=True)
+
+    style = InstanceDict(SliderStyle).tag(sync=True, **widget_serialization)
+
+
+@register
 class FloatProgress(_BoundedFloat):
     """ Displays a progress bar.
 
@@ -156,8 +232,6 @@ class FloatProgress(_BoundedFloat):
         minimal position of the slider
     max : float
         maximal position of the slider
-    step : float
-        step of the progress bar
     description : str
         name of the progress bar
     orientation : {'horizontal', 'vertical'}
@@ -167,14 +241,16 @@ class FloatProgress(_BoundedFloat):
         colors are: 'success'-green, 'info'-light blue, 'warning'-orange, 'danger'-red
     """
     _view_name = Unicode('ProgressView').tag(sync=True)
-    _model_name = Unicode('ProgressModel').tag(sync=True)
+    _model_name = Unicode('FloatProgressModel').tag(sync=True)
     orientation = CaselessStrEnum(values=['horizontal', 'vertical'],
         default_value='horizontal', help="Vertical or horizontal.").tag(sync=True)
 
     bar_style = CaselessStrEnum(
         values=['success', 'info', 'warning', 'danger', ''],
-        default_value='', allow_none=True, help="""Use a predefined styling for
-        the progess bar.""").tag(sync=True)
+        default_value='', allow_none=True,
+        help="Use a predefined styling for the progess bar.").tag(sync=True)
+
+    style = InstanceDict(ProgressStyle).tag(sync=True, **widget_serialization)
 
 
 class _FloatRange(_Float):
@@ -212,7 +288,7 @@ class _BoundedFloatRange(_FloatRange):
 
     def __init__(self, *args, **kwargs):
         min, max = kwargs.get('min', 0.0), kwargs.get('max', 100.0)
-        if not kwargs.get('value', None):
+        if kwargs.get('value', None) is None:
             kwargs['value'] = (0.75 * min + 0.25 * max,
                                0.25 * min + 0.75 * max)
         super(_BoundedFloatRange, self).__init__(*args, **kwargs)
@@ -239,7 +315,7 @@ class _BoundedFloatRange(_FloatRange):
         return lower, upper
 
 
-@register('Jupyter.FloatRangeSlider')
+@register
 class FloatRangeSlider(_BoundedFloatRange):
     """ Slider/trackbar that represents a pair of floats bounded by minimum and maximum value.
 
@@ -263,16 +339,16 @@ class FloatRangeSlider(_BoundedFloatRange):
         default is '.2f', specifier for the format function used to represent
         slider value for human consumption, modeled after Python 3's format
         specification mini-language (PEP 3101).
-    slider_color : str Unicode color code (eg. '#C13535')
-        color of the slider
-    color : str Unicode color code (eg. '#C13535')
-        color of the value displayed (if readout == True)
     """
-    _view_name = Unicode('FloatSliderView').tag(sync=True)
-    _model_name = Unicode('FloatSliderModel').tag(sync=True)
+    _view_name = Unicode('FloatRangeSliderView').tag(sync=True)
+    _model_name = Unicode('FloatRangeSliderModel').tag(sync=True)
+    step = CFloat(0.1, help="Minimum step to increment the value").tag(sync=True)
     orientation = CaselessStrEnum(values=['horizontal', 'vertical'],
         default_value='horizontal', help="Vertical or horizontal.").tag(sync=True)
-    _range = Bool(True, help="Display a range selector").tag(sync=True)
     readout = Bool(True, help="Display the current value of the slider next to it.").tag(sync=True)
-    slider_color = Color(None, allow_none=True).tag(sync=True)
+    readout_format = NumberFormat(
+        '.2f', help="Format for the readout").tag(sync=True)
     continuous_update = Bool(True, help="Update the value of the widget as the user is sliding the slider.").tag(sync=True)
+    disabled = Bool(False, help="Enable or disable user changes").tag(sync=True)
+
+    style = InstanceDict(SliderStyle).tag(sync=True, **widget_serialization)
